@@ -30,6 +30,8 @@ pub mod analysis;
 pub mod docs;
 
 pub mod coverage;
+#[cfg(feature = "cli")]
+pub mod debug;
 
 use crate::clarity::callables::CallableType;
 use crate::clarity::contexts::GlobalContext;
@@ -219,6 +221,12 @@ pub fn eval<'a>(
         Atom, AtomValue, Field, List, LiteralValue, TraitReference,
     };
 
+    #[cfg(feature = "cli")]
+    if let Some(mut debug_state) = env.global_context.debug_state.take() {
+        debug_state.begin_eval(env, context, exp);
+        env.global_context.debug_state = Some(debug_state);
+    }
+
     if let Some(ref mut coverage_tracker) = env.global_context.coverage_reporting {
         coverage_tracker.report_eval(&env.contract_context.contract_identifier, exp);
     }
@@ -260,6 +268,13 @@ pub fn eval<'a>(
             _ => {}
         };
     }
+
+    #[cfg(feature = "cli")]
+    if let Some(mut debug_state) = env.global_context.debug_state.take() {
+        debug_state.finish_eval(env, context, exp, &res);
+        env.global_context.debug_state = Some(debug_state);
+    }
+
     res
 }
 
