@@ -64,6 +64,7 @@ pub struct Session {
     pub costs_reports: Vec<CostsReport>,
     pub initial_contracts_analysis: Vec<(ContractAnalysis, String, String)>,
     pub show_costs: bool,
+    pub executed: Vec<String>,
 }
 
 impl Session {
@@ -90,6 +91,7 @@ impl Session {
             initial_contracts_analysis: vec![],
             show_costs: false,
             settings,
+            executed: Vec::new(),
         }
     }
 
@@ -335,6 +337,7 @@ impl Session {
                     code.to_string(),
                     Some(contract_name.to_string()),
                     true,
+                    false,
                     None,
                 ) {
                     Ok((mut logs, _)) => output.append(&mut logs),
@@ -378,6 +381,7 @@ impl Session {
                     contract.code,
                     contract.name,
                     true,
+                    false,
                     Some("Deployment".into()),
                 ) {
                     Ok((ref mut res_output, result)) => {
@@ -430,6 +434,7 @@ impl Session {
                 POX_CONTRACT.to_string(),
                 Some("pox".to_string()),
                 false,
+                false,
                 None,
             )
             .expect("Unable to deploy POX");
@@ -443,6 +448,7 @@ impl Session {
                 BNS_CONTRACT.to_string(),
                 Some("bns".to_string()),
                 false,
+                false,
                 None,
             )
             .expect("Unable to deploy BNS");
@@ -456,6 +462,7 @@ impl Session {
                 COSTS_V1_CONTRACT.to_string(),
                 Some("costs-v1".to_string()),
                 false,
+                false,
                 None,
             )
             .expect("Unable to deploy COSTS");
@@ -469,6 +476,7 @@ impl Session {
                 COSTS_V2_CONTRACT.to_string(),
                 Some("costs-v2".to_string()),
                 false,
+                false,
                 None,
             )
             .expect("Unable to deploy COSTS");
@@ -532,6 +540,8 @@ impl Session {
             cmd if cmd.starts_with("::toggle_costs") => self.toggle_costs(&mut output),
             cmd if cmd.starts_with("::encode") => self.encode(&mut output, cmd),
             cmd if cmd.starts_with("::decode") => self.decode(&mut output, cmd),
+            #[cfg(feature = "cli")]
+            cmd if cmd.starts_with("::debug") => self.debug(&mut output, cmd),
 
             snippet => {
                 if self.show_costs {
@@ -541,6 +551,7 @@ impl Session {
                         snippet.to_string(),
                         None,
                         true,
+                        false,
                         None,
                     ) {
                         Ok((mut output, result)) => {
@@ -564,11 +575,18 @@ impl Session {
         snippet: String,
         name: Option<String>,
         cost_track: bool,
+        debug: bool,
         test_name: Option<String>,
     ) -> Result<(Vec<String>, ExecutionResult), Vec<String>> {
         let light_red = Colour::Red.bold();
 
-        let result = self.interpret(snippet.to_string(), name.clone(), cost_track, test_name);
+        let result = self.interpret(
+            snippet.to_string(),
+            name.clone(),
+            cost_track,
+            debug,
+            test_name,
+        );
         let mut output = Vec::<String>::new();
         let lines = snippet.lines();
         let formatted_lines: Vec<String> = lines.map(|l| l.to_string()).collect();
@@ -599,6 +617,31 @@ impl Session {
         }
     }
 
+    #[cfg(feature = "cli")]
+    pub fn debug(&mut self, output: &mut Vec<String>, cmd: &str) {
+        let snippet = match cmd.split_once(" ") {
+            Some((_, snippet)) => snippet,
+            _ => return output.push(red!("Usage: ::debug <expr>")),
+        };
+        let mut result = match self.formatted_interpretation(
+            snippet.to_string(),
+            None,
+            true,
+            true,
+            None,
+        ) {
+            Ok((mut output, result)) => {
+                if let Some((ref contract_name, _, _, _, _)) = result.contract {
+                    let snippet = format!("→ .{} contract successfully stored. Use (contract-call? ...) for invoking the public functions:", contract_name.clone());
+                    output.push(green!(snippet));
+                }
+                output
+            }
+            Err(result) => result,
+        };
+        output.append(&mut result);
+    }
+
     pub fn invoke_contract_call(
         &mut self,
         contract: &str,
@@ -624,7 +667,7 @@ impl Session {
         );
 
         self.set_tx_sender(sender.into());
-        let result = self.interpret(snippet, None, true, Some(test_name.clone()))?;
+        let result = self.interpret(snippet, None, true, false, Some(test_name.clone()))?;
         if let Some(ref cost) = result.cost {
             self.costs_reports.push(CostsReport {
                 test_name,
@@ -643,6 +686,7 @@ impl Session {
         snippet: String,
         name: Option<String>,
         cost_track: bool,
+        debug: bool,
         test_name: Option<String>,
     ) -> Result<ExecutionResult, Vec<Diagnostic>> {
         let (contract_name, is_tx) = match name {
@@ -667,10 +711,13 @@ impl Session {
             QualifiedContractIdentifier::parse(&id).unwrap()
         };
 
-        match self
-            .interpreter
-            .run(snippet, contract_identifier.clone(), cost_track, report)
-        {
+        match self.interpreter.run(
+            snippet,
+            contract_identifier.clone(),
+            cost_track,
+            debug,
+            report,
+        ) {
             Ok(result) => {
                 if let Some(ref coverage) = result.coverage {
                     self.coverage_reports.push(coverage.clone());
@@ -847,7 +894,7 @@ impl Session {
             _ => return output.push(red!("Usage: ::encode <expr>")),
         };
 
-        let result = self.interpret(snippet.to_string(), None, false, None);
+        let result = self.interpret(snippet.to_string(), None, false, false, None);
         let value = match result {
             Ok(result) => {
                 let mut tx_bytes = vec![];
@@ -896,10 +943,11 @@ impl Session {
     #[cfg(feature = "cli")]
     pub fn get_costs(&mut self, output: &mut Vec<String>, cmd: &str) {
         let snippet = cmd.to_string().split_off("::get_costs ".len());
-        let (mut result, cost) = match self.formatted_interpretation(snippet, None, true, None) {
-            Ok((output, result)) => (output, result.cost.clone()),
-            Err(output) => (output, None),
-        };
+        let (mut result, cost) =
+            match self.formatted_interpretation(snippet, None, true, false, None) {
+                Ok((output, result)) => (output, result.cost.clone()),
+                Err(output) => (output, None),
+            };
 
         if let Some(cost) = cost {
             let headers = vec!["".to_string(), "Consumed".to_string(), "Limit".to_string()];
@@ -1006,10 +1054,11 @@ impl Session {
     #[cfg(not(feature = "cli"))]
     pub fn get_costs(&mut self, output: &mut Vec<String>, cmd: &str) {
         let snippet = cmd.to_string().split_off("::get_costs ".len());
-        let (mut result, cost) = match self.formatted_interpretation(snippet, None, true, None) {
-            Ok((output, result)) => (output, result.cost.clone()),
-            Err(output) => (output, None),
-        };
+        let (mut result, cost) =
+            match self.formatted_interpretation(snippet, None, true, false, None) {
+                Ok((output, result)) => (output, result.cost.clone()),
+                Err(output) => (output, None),
+            };
 
         if let Some(cost) = cost {
             output.push(format!(
